import { ArgumentInvalidException } from '@mobicoop/ddd-library';
import { Role } from '@modules/ad/core/domain/ad.types';
import { Target } from '@modules/ad/core/domain/candidate.types';
import { ActorTime } from '@modules/ad/core/domain/value-objects/actor-time.value-object';
import { JourneyItem } from '@modules/ad/core/domain/value-objects/journey-item.value-object';
import { Journey } from '@modules/ad/core/domain/value-objects/journey.value-object';

describe('Journey value object', () => {
  it('should create a journey value object', () => {
    const journeyVO = new Journey({
      firstDate: new Date('2023-09-01'),
      lastDate: new Date('2024-08-30'),
      journeyItems: [
        new JourneyItem({
          lat: 48.689445,
          lon: 6.17651,
          duration: 0,
          distance: 0,
          actorTimes: [
            new ActorTime({
              role: Role.DRIVER,
              target: Target.START,
              firstDatetime: new Date('2023-09-01 07:00'),
              firstMinDatetime: new Date('2023-09-01 06:45'),
              firstMaxDatetime: new Date('2023-09-01 07:15'),
              lastDatetime: new Date('2024-08-30 07:00'),
              lastMinDatetime: new Date('2024-08-30 06:45'),
              lastMaxDatetime: new Date('2024-08-30 07:15'),
            }),
          ],
        }),
        new JourneyItem({
          lat: 48.369445,
          lon: 6.67487,
          duration: 2100,
          distance: 56878,
          actorTimes: [
            new ActorTime({
              role: Role.DRIVER,
              target: Target.NEUTRAL,
              firstDatetime: new Date('2023-09-01 07:35'),
              firstMinDatetime: new Date('2023-09-01 07:20'),
              firstMaxDatetime: new Date('2023-09-01 07:50'),
              lastDatetime: new Date('2024-08-30 07:35'),
              lastMinDatetime: new Date('2024-08-30 07:20'),
              lastMaxDatetime: new Date('2024-08-30 07:50'),
            }),
            new ActorTime({
              role: Role.PASSENGER,
              target: Target.START,
              firstDatetime: new Date('2023-09-01 07:32'),
              firstMinDatetime: new Date('2023-09-01 07:17'),
              firstMaxDatetime: new Date('2023-09-01 07:47'),
              lastDatetime: new Date('2024-08-30 07:32'),
              lastMinDatetime: new Date('2024-08-30 07:17'),
              lastMaxDatetime: new Date('2024-08-30 07:47'),
            }),
          ],
        }),
        new JourneyItem({
          lat: 47.98487,
          lon: 6.9427,
          duration: 3840,
          distance: 76491,
          actorTimes: [
            new ActorTime({
              role: Role.DRIVER,
              target: Target.NEUTRAL,
              firstDatetime: new Date('2023-09-01 08:04'),
              firstMinDatetime: new Date('2023-09-01 07:51'),
              firstMaxDatetime: new Date('2023-09-01 08:19'),
              lastDatetime: new Date('2024-08-30 08:04'),
              lastMinDatetime: new Date('2024-08-30 07:51'),
              lastMaxDatetime: new Date('2024-08-30 08:19'),
            }),
            new ActorTime({
              role: Role.PASSENGER,
              target: Target.FINISH,
              firstDatetime: new Date('2023-09-01 08:01'),
              firstMinDatetime: new Date('2023-09-01 07:46'),
              firstMaxDatetime: new Date('2023-09-01 08:16'),
              lastDatetime: new Date('2024-08-30 08:01'),
              lastMinDatetime: new Date('2024-08-30 07:46'),
              lastMaxDatetime: new Date('2024-08-30 08:16'),
            }),
          ],
        }),
        new JourneyItem({
          lat: 47.365987,
          lon: 7.02154,
          duration: 4980,
          distance: 96475,
          actorTimes: [
            new ActorTime({
              role: Role.DRIVER,
              target: Target.FINISH,
              firstDatetime: new Date('2023-09-01 08:23'),
              firstMinDatetime: new Date('2023-09-01 08:08'),
              firstMaxDatetime: new Date('2023-09-01 08:38'),
              lastDatetime: new Date('2024-08-30 08:23'),
              lastMinDatetime: new Date('2024-08-30 08:08'),
              lastMaxDatetime: new Date('2024-08-30 08:38'),
            }),
          ],
        }),
      ],
    });
    expect(journeyVO.journeyItems).toHaveLength(4);
    expect(journeyVO.firstDate.getUTCDate()).toBe(1);
    expect(journeyVO.lastDate.getUTCMonth()).toBe(7);
  });
  it('should throw an error if dates are inconsistent', () => {
    expect(
      () =>
        new Journey({
          firstDate: new Date('2023-09-01'),
          lastDate: new Date('2024-08-31'),
          journeyItems: [
            new JourneyItem({
              lat: 48.689445,
              lon: 6.17651,
              duration: 0,
              distance: 0,
              actorTimes: [
                new ActorTime({
                  role: Role.DRIVER,
                  target: Target.START,
                  firstDatetime: new Date('2023-09-01 07:00'),
                  firstMinDatetime: new Date('2023-09-01 06:45'),
                  firstMaxDatetime: new Date('2023-09-01 07:15'),
                  lastDatetime: new Date('2024-08-30 07:00'),
                  lastMinDatetime: new Date('2024-08-30 06:45'),
                  lastMaxDatetime: new Date('2024-08-30 07:15'),
                }),
              ],
            }),
            new JourneyItem({
              lat: 48.369445,
              lon: 6.67487,
              duration: 2100,
              distance: 56878,
              actorTimes: [
                new ActorTime({
                  role: Role.DRIVER,
                  target: Target.NEUTRAL,
                  firstDatetime: new Date('2023-09-01 07:35'),
                  firstMinDatetime: new Date('2023-09-01 07:20'),
                  firstMaxDatetime: new Date('2023-09-01 07:50'),
                  lastDatetime: new Date('2024-08-30 07:35'),
                  lastMinDatetime: new Date('2024-08-30 07:20'),
                  lastMaxDatetime: new Date('2024-08-30 07:50'),
                }),
                new ActorTime({
                  role: Role.PASSENGER,
                  target: Target.START,
                  firstDatetime: new Date('2023-09-01 07:32'),
                  firstMinDatetime: new Date('2023-09-01 07:17'),
                  firstMaxDatetime: new Date('2023-09-01 07:47'),
                  lastDatetime: new Date('2024-08-30 07:32'),
                  lastMinDatetime: new Date('2024-08-30 07:17'),
                  lastMaxDatetime: new Date('2024-08-30 07:47'),
                }),
              ],
            }),
            new JourneyItem({
              lat: 47.98487,
              lon: 6.9427,
              duration: 3840,
              distance: 76491,
              actorTimes: [
                new ActorTime({
                  role: Role.DRIVER,
                  target: Target.NEUTRAL,
                  firstDatetime: new Date('2023-09-01 08:04'),
                  firstMinDatetime: new Date('2023-09-01 07:51'),
                  firstMaxDatetime: new Date('2023-09-01 08:19'),
                  lastDatetime: new Date('2024-08-30 08:04'),
                  lastMinDatetime: new Date('2024-08-30 07:51'),
                  lastMaxDatetime: new Date('2024-08-30 08:19'),
                }),
                new ActorTime({
                  role: Role.PASSENGER,
                  target: Target.FINISH,
                  firstDatetime: new Date('2023-09-01 08:01'),
                  firstMinDatetime: new Date('2023-09-01 07:46'),
                  firstMaxDatetime: new Date('2023-09-01 08:16'),
                  lastDatetime: new Date('2024-08-30 08:01'),
                  lastMinDatetime: new Date('2024-08-30 07:46'),
                  lastMaxDatetime: new Date('2024-08-30 08:16'),
                }),
              ],
            }),
            new JourneyItem({
              lat: 47.365987,
              lon: 7.02154,
              duration: 4980,
              distance: 96475,
              actorTimes: [
                new ActorTime({
                  role: Role.DRIVER,
                  target: Target.FINISH,
                  firstDatetime: new Date('2023-09-01 08:23'),
                  firstMinDatetime: new Date('2023-09-01 08:08'),
                  firstMaxDatetime: new Date('2023-09-01 08:38'),
                  lastDatetime: new Date('2024-08-30 08:23'),
                  lastMinDatetime: new Date('2024-08-30 08:08'),
                  lastMaxDatetime: new Date('2024-08-30 08:38'),
                }),
              ],
            }),
          ],
        }),
    ).toThrow(ArgumentInvalidException);
    expect(
      () =>
        new Journey({
          firstDate: new Date('2024-08-30'),
          lastDate: new Date('2023-09-01'),
          journeyItems: [
            new JourneyItem({
              lat: 48.689445,
              lon: 6.17651,
              duration: 0,
              distance: 0,
              actorTimes: [
                new ActorTime({
                  role: Role.DRIVER,
                  target: Target.START,
                  firstDatetime: new Date('2023-09-01 07:00'),
                  firstMinDatetime: new Date('2023-09-01 06:45'),
                  firstMaxDatetime: new Date('2023-09-01 07:15'),
                  lastDatetime: new Date('2024-08-30 07:00'),
                  lastMinDatetime: new Date('2024-08-30 06:45'),
                  lastMaxDatetime: new Date('2024-08-30 07:15'),
                }),
              ],
            }),
            new JourneyItem({
              lat: 48.369445,
              lon: 6.67487,
              duration: 2100,
              distance: 56878,
              actorTimes: [
                new ActorTime({
                  role: Role.DRIVER,
                  target: Target.NEUTRAL,
                  firstDatetime: new Date('2023-09-01 07:35'),
                  firstMinDatetime: new Date('2023-09-01 07:20'),
                  firstMaxDatetime: new Date('2023-09-01 07:50'),
                  lastDatetime: new Date('2024-08-30 07:35'),
                  lastMinDatetime: new Date('2024-08-30 07:20'),
                  lastMaxDatetime: new Date('2024-08-30 07:50'),
                }),
                new ActorTime({
                  role: Role.PASSENGER,
                  target: Target.START,
                  firstDatetime: new Date('2023-09-01 07:32'),
                  firstMinDatetime: new Date('2023-09-01 07:17'),
                  firstMaxDatetime: new Date('2023-09-01 07:47'),
                  lastDatetime: new Date('2024-08-30 07:32'),
                  lastMinDatetime: new Date('2024-08-30 07:17'),
                  lastMaxDatetime: new Date('2024-08-30 07:47'),
                }),
              ],
            }),
            new JourneyItem({
              lat: 47.98487,
              lon: 6.9427,
              duration: 3840,
              distance: 76491,
              actorTimes: [
                new ActorTime({
                  role: Role.DRIVER,
                  target: Target.NEUTRAL,
                  firstDatetime: new Date('2023-09-01 08:04'),
                  firstMinDatetime: new Date('2023-09-01 07:51'),
                  firstMaxDatetime: new Date('2023-09-01 08:19'),
                  lastDatetime: new Date('2024-08-30 08:04'),
                  lastMinDatetime: new Date('2024-08-30 07:51'),
                  lastMaxDatetime: new Date('2024-08-30 08:19'),
                }),
                new ActorTime({
                  role: Role.PASSENGER,
                  target: Target.FINISH,
                  firstDatetime: new Date('2023-09-01 08:01'),
                  firstMinDatetime: new Date('2023-09-01 07:46'),
                  firstMaxDatetime: new Date('2023-09-01 08:16'),
                  lastDatetime: new Date('2024-08-30 08:01'),
                  lastMinDatetime: new Date('2024-08-30 07:46'),
                  lastMaxDatetime: new Date('2024-08-30 08:16'),
                }),
              ],
            }),
            new JourneyItem({
              lat: 47.365987,
              lon: 7.02154,
              duration: 4980,
              distance: 96475,
              actorTimes: [
                new ActorTime({
                  role: Role.DRIVER,
                  target: Target.FINISH,
                  firstDatetime: new Date('2023-09-01 08:23'),
                  firstMinDatetime: new Date('2023-09-01 08:08'),
                  firstMaxDatetime: new Date('2023-09-01 08:38'),
                  lastDatetime: new Date('2024-08-30 08:23'),
                  lastMinDatetime: new Date('2024-08-30 08:08'),
                  lastMaxDatetime: new Date('2024-08-30 08:38'),
                }),
              ],
            }),
          ],
        }),
    ).toThrow(ArgumentInvalidException);
  });
  it('should throw an error if journeyItems is too short', () => {
    expect(
      () =>
        new Journey({
          firstDate: new Date('2023-09-01'),
          lastDate: new Date('2024-08-30'),
          journeyItems: [
            new JourneyItem({
              lat: 48.689445,
              lon: 6.17651,
              duration: 0,
              distance: 0,
              actorTimes: [
                new ActorTime({
                  role: Role.DRIVER,
                  target: Target.START,
                  firstDatetime: new Date('2023-09-01 07:00'),
                  firstMinDatetime: new Date('2023-09-01 06:45'),
                  firstMaxDatetime: new Date('2023-09-01 07:15'),
                  lastDatetime: new Date('2024-08-30 07:00'),
                  lastMinDatetime: new Date('2024-08-30 06:45'),
                  lastMaxDatetime: new Date('2024-08-30 07:15'),
                }),
              ],
            }),
          ],
        }),
    ).toThrow(ArgumentInvalidException);
  });
});
